unit uTestSimplex;
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  xMatDefs,
  uSimplex, uMIPSImplex,
  uConstantesSimSEE, uListaViolacionesPermitidasSimplex;

const
  WM_DBGMONSPX = WM_USER + 1;

type

  { TformTestSimplex }

  TformTestSimplex = class(TForm)
    btAbrir: TButton;
    BTCerrar: TButton;
    OpenDialog1: TOpenDialog;
    BTActualizar: TButton;
    memo: TMemo;
    btContinuar: TButton;
    eCntLlamadas: TEdit;
    eCntHasta: TEdit;
    procedure btAbrirClick(Sender: TObject);
    procedure BTCerrarClick(Sender: TObject);
    procedure BTActualizarClick(Sender: TObject);
    procedure btContinuarClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    procedure CargarArchivoFromXLT(archi: string);
    procedure Procesar_DBGMONSPX(var Message: TMessage); message WM_DBGMONSPX;
    procedure ResolverReferenciasYCargarSimplex( archi: string );
  public
    spx: TMipSimplex;
  end;

var
  formTestSimplex: TformTestSimplex;

implementation
{$R *.lfm}

procedure TformTestSimplex.CargarArchivoFromXLT(archi: string);
begin
  btAbrir.Enabled := False;
  ResolverReferenciasYCargarSimplex( archi );
  spx.Resolver;
end;

procedure TformTestSimplex.Procesar_DBGMONSPX(var Message: TMessage);
var
  ts: string;
  cntLLamadas, cntHasta: integer;
begin
  cntLlamadas := StrToInt(ecntLlamadas.Text);
  Inc(cntLlamadas);
  ecntLlamadas.Text := IntToStr(cntLlamadas);
  cntHasta := StrToInt(ecntHasta.Text);
  ts := ecntLlamadas.Text + '::>' + dbgmonspx_LeerMensaje;
  memo.Lines.Append(ts);
  if cntLlamadas <= cntHasta then
    btContinuarClick(self)
  else
    btContinuar.Enabled := True;
end;

procedure TformTestSimplex.btAbrirClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    abrirExcel(OpenDialog1.FileName);
  end;
end;

procedure TformTestSimplex.BTCerrarClick(Sender: TObject);
begin
  if assigned(xf) then
  begin
    xf.Free;
    xf := nil;
    dbgmonspx_Abortar;
  end;
  btAbrir.Enabled := True;
end;

procedure TformTestSimplex.ResolverReferenciasYCargarSimplex(archi: string);
var
  f: textfile;
begin
  filemode:= 0;
  assignfile( f, archi );
  reset( f );
  spx:= TMIPSimplex.CreateFromXLT( f );
  closefile( f );
end;

procedure TformTestSimplex.BTActualizarClick(Sender: TObject);
var
  k, j: integer;
  spx: TSimplex;
  cursorAnterior: TCursor;
begin
  spx := umipsimplex.spxActivo;
  if spx = nil then
    exit;
  BTContinuar.Enabled := False;
  cursorAnterior := screen.Cursor;
  Screen.Cursor := crHourGlass;

  xf.Write(kfilx - 3, 2, spx.cnt_varfijas);
  xf.Write(kfilx - 2, 2, spx.cnt_RestriccionesRedundantes_);
  //Cotas y flgs_x
  for j := 1 to spx.nc - 1 do
  begin
    xf.Write(kfilx + 1, j + 1, spx.x_inf.pv[j]);
    xf.Write(kfilx + 2, j + 1, spx.x_sup.pv[j]);
    xf.Write(kfilx + 3, j + 1, spx.flg_x[j]);
  end;

  //flgs_y
  for j := 1 to spx.nf - 1 do
  begin
    xf.Write(kfilx + 4, j + 1, spx.flg_y[j]);
  end;

  //Top y left
  for j := 1 to spx.nc - 1 do
    xf.Write(kfilx + 5, j + 1, spx.top[j]);
  for j := 1 to spx.nf - 1 do
    xf.Write(kfilx + 6, j + 1, spx.left[j]);

  //Encabezados de las variables
  for j := 1 to spx.nc do
    if spx.top[j] < 0 then
      xf.Write(kfilx + 10 + 0, j + 1, spx.nombreVars[-spx.top[j]])
    else
      xf.Write(kfilx + 10 + 0, j + 1, spx.nombreRest[spx.top[j]]);

  //Nombres de las restricciones y valores que tienen que cumplir
  for k := 1 to spx.nf - 1 do
    if spx.left[k] > 0 then
    begin
      //Es una y
      xf.Write(kfilx + 10 + k, 1, spx.nombreRest[spx.left[k]]);
      case spx.flg_y[spx.left[k]] of
        0:
        begin
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '>= 0');
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 2, '');
        end;
        2:
        begin
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '= 0');
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '');
        end;
        -2:
        begin
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '= 0');
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '');
        end;
        else
          //Cota superior de una restriccion??
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '??, flg_y=' + IntToStr(spx.flg_y[spx.left[k]]));
      end; // case
    end
    else
    begin
      //Es una x
      xf.Write(kfilx + 10 + k, 1, spx.nombreVars[-spx.left[k]]);
      case spx.flg_x[-spx.left[k]] of
        -1, 1:
        begin
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '>= 0');
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 2, '<= ' +
            FloatToStr(spx.x_sup.e(-spx.left[k])));
        end;
        -2, 2:
        begin
          //                if pos('=', h.cells[kfilx+10+k, 1+spx.nc+2]) <> 0 then
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 2, ' ');
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '= 0');
          // + FloatToStr(spx.x_inf.e(-spx.left[k]));
        end;
        0:
        begin
          //            if pos('=', h.cells[kfilx+10+k, 1+spx.nc+2]) <> 0 then
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 2, ' ');
          xf.Write(kfilx + 10 + k, 1 + spx.nc + 1, '>= 0');
        end
      end; //  case
    end;
  xf.Write(kfilx + 10 + spx.nf, 1, 'z');

  for k := 1 to spx.nf do
    for j := 1 to spx.nc do
    begin
      //      h.cells[kfilx+10+k, j+1]:= spx.e(k, j);
      xf.Write(kfilx + 10 + k, j + 1, spx.e(k, j));
      if abs(spx.e(k, j)) > 1E15 then
        memo.Lines.Add('OJO: abs(spx[' + IntToStr(k) + ', ' + IntToStr(j) + ']) > 1E15');
    end;
  BTContinuar.Enabled := True;
  Screen.Cursor := cursorAnterior;
end;

procedure TformTestSimplex.btContinuarClick(Sender: TObject);
begin
  btContinuar.Enabled := False;
  dbgmonspx_Continuar;
end;

procedure TformTestSimplex.FormCreate(Sender: TObject);
begin
  usimplex.dbgmonspx_RegistrarMonitor(Handle);
  xf := nil;
  OpenDialog1.InitialDir := getDir_Dbg;
  if ParamCount >= 1 then
  begin
    if FileExists(ParamStr(1)) then
    begin
      OpenDialog1.FileName := ParamStr(1);
      //    abrirExcel(OpenDialog1.FileName);
      if ParamCount >= 2 then
      begin
        try
          StrToInt(ParamStr(2));
          eCntHasta.Text := ParamStr(2);
        except
          //Ignoro el parámetro
        end;
      end;
    end;
  end;
end;

procedure TformTestSimplex.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  BTCerrarClick(BTCerrar);
  if xf <> nil then
  begin
    xf.Free;
    xf := nil;
    dbgmonspx_RegistrarMonitor(0);
    dbgmonspx_Continuar;
  end;
end;

end.
